@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2012-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2012-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 
 from html import escape
@@ -20,13 +20,14 @@ from lino.core import choicelists
 from lino.core.gfks import ContentType
 from lino.modlib.extjs.ext_renderer import ExtRenderer
 
-from lino.core.actions import (ShowEmptyTable, ShowDetail,
+from lino.core.actions import (ShowEmptyTable, ShowDetail, QuickLinkAction,
                                ShowInsert, ShowTable, SubmitDetail,
                                SubmitInsert, Action)
 from lino.core.boundaction import BoundAction
 from lino.core.choicelists import ChoiceListMeta
+from lino.modlib.system.choicelists import DashboardLayouts
 from lino.core.actors import Actor
-from lino.core.layouts import LayoutHandle
+from lino.core.layouts import LayoutHandle, BaseLayout
 from lino.core.elems import LayoutElement, ComboFieldElement, SimpleRemoteComboFieldElement, FieldElement
 from lino.core import kernel
 
@@ -73,11 +74,11 @@ class Renderer(JsRenderer, JsCacheRenderer):
 
     def write_lino_js(self, f):
         """
+        Creates what is known as window.App.state.site_data in React code.
 
         :param f: File object
         :return: 1
         """
-        self.serialise_js_code = True
         choicelists_data = {
             # ID: [{"value": py2js(c[0]).strip('"'), "text": py2js(c[1]).strip('"')} for c in cl.get_choices()] for
             ID: [{"value": c[0].value, "text": str(c[1])} for c in cl.get_choices()]
@@ -90,17 +91,77 @@ class Renderer(JsRenderer, JsCacheRenderer):
                 if ba.action not in actions:
                     actions.add(ba.action)
 
-        f.write(py2js(dict(
+        self.serialise_js_code = True
+        plugins = settings.SITE.plugins
+        data = dict(
             actions={a.action_name: a for a in actions},
             # actors={a.actor_id: a for a in self.actors_list},
             menu=settings.SITE.get_site_menu(get_user_profile()),
             choicelists=choicelists_data,
-            suggestors=list(settings.SITE.plugins.memo.parser.suggesters.keys())  # [#,@] keytriggers
-        ),
-            compact=not settings.SITE.is_demo_site))
+            suggestors=list(plugins.memo.parser.suggesters.keys()),  # [#,@] keytriggers
+            )
+
+        if settings.SITE.is_installed('notify'):
+            data.update(vapid_public_key=plugins.notify.vapid_public_key)
+
+        # data.update(actors={str(a): a for a in self.actors_list})
+        # data.update(action_param_panels={
+        #     str(p): self.panel2json(p)
+        #         for p in self.action_param_panels })
+        # data.update(param_panels={
+        #     str(p): self.panel2json(p)
+        #         for p in self.param_panels })
+        form_panels = {}
+        for p in self.form_panels:
+            form_panels[p._formpanel_name] = self.panel2json(p)
+        for p in self.param_panels:
+            form_panels[p._formpanel_name] = self.panel2json(p)
+        for p in self.action_param_panels:
+            form_panels[p._formpanel_name] = self.panel2json(p)
+        for p in self.other_panels:
+            form_panels[p._formpanel_name] = self.panel2json(p)
+        data.update(form_panels=form_panels)
+
+
+        # print("20210417", data)
+
+        f.write(py2js(data))
+        # f.write(py2js(data, compact=not settings.SITE.is_demo_site))
         self.serialise_js_code = False
         return 1
 
+    def panel2json(self, p):
+        assert isinstance(p, BaseLayout)
+        # is a Layout instance
+        # return ["foo"]
+        v = p.get_layout_handle()
+        return dict(main=self.elem2json(v.main),
+                    window_size=v.layout.window_size)
+
+    def elem2json(self, v):
+        assert isinstance(v, LayoutElement)
+        # Layout elems
+        result = dict(label=v.get_label(),
+                      # repr=repr(v),
+                      react_name=v.__class__.__name__)  # Used for choosing correct react component
+        if hasattr(v, "elements"):  # dd
+            result['items'] = [self.elem2json(e) for e in v.elements if e.is_visible()]
+            # result['items'] = [e for e in v.elements if e.get_view_permission(get_user_profile())]
+        result.update(obj2dict(v, "fields_index fields_index_hidden editable vertical hpad is_fieldset name width preferred_width\
+                                  hidden value hflex vflex"))
+        # result["width"] = v.width or v.preferred_width
+        # Slave tables
+        if hasattr(v, "actor"):
+            # reference to actor data for slave-grids
+            result.update(obj2dict(v.actor, "actor_id"))  # to get siteDate layout index
+
+        # if hasattr(v, "get_field_options"):
+        if issubclass(v.__class__, FieldElement):
+            result.update(field_options=v.get_field_options())
+            result.update(help_text=v.field.help_text)
+
+        return result
+
     # working, but shouldn't be used, as it clears the app history
 
     def reload_js(self):
@@ -146,7 +207,7 @@ class Renderer(JsRenderer, JsCacheRenderer):
             return self.window_action_button(ar, ba, st, label, **kw)
         return self.row_action_button(obj, ar, ba, label, **kw)
 
-    # from extrendere
+    # from ext_renderer
     def action_call_on_instance(
             self, obj, ar, ba, request_kwargs={}, **status):
         """Note that `ba.actor` may differ from `ar.actor` when defined on a
@@ -158,7 +219,7 @@ class Renderer(JsRenderer, JsCacheRenderer):
         :ba:  The bound action
         :request_kwargs: keyword arguments to forwarded to the child action request
 
-        Any kwyword other arguments are forwarded to :meth:`ar2js`.
+        Any keyword other arguments are forwarded to :meth:`ar2js`.
 
         """
         if ar is None:
@@ -228,6 +289,8 @@ class Renderer(JsRenderer, JsCacheRenderer):
         """
         if v is settings.SITE.LANGUAGE_CHOICES:
             return js_code('LANGUAGE_CHOICES')
+        if isinstance(v, LayoutHandle):
+            raise Exception("20210517 {}".format(v))
         if isinstance(v, choicelists.Choice):
             """
             This is special. We don't render the text but the value.
@@ -278,31 +341,6 @@ class Renderer(JsRenderer, JsCacheRenderer):
                     # ~ handler=js_code("function() { window.location='%s'; }" % url))
                     handler=js_code("function() { Lino.load_url('%s'); }" % url))
             return dict(text=v.label, href=url)
-        if issubclass(v.__class__, LayoutElement):
-            # Layout elems
-            result = dict(label=v.get_label(),
-                          repr=repr(v),
-                          react_name=v.__class__.__name__)  # Used for choosing correct react component
-            if hasattr(v, "elements"):  # dd
-                result['items'] = [e for e in v.elements if e.get_view_permission(get_user_profile())]
-            result.update(obj2dict(v, "fields_index fields_index_hidden editable vertical hpad is_fieldset name width preferred_width\
-                                      hidden value hflex vflex"))
-            # result["width"] = v.width or v.preferred_width
-            # Slave tables
-            if hasattr(v, "actor"):
-                # reference to actor data for slave-grids
-                result.update(obj2dict(v.actor, "actor_id"))  # to get siteDate layout index
-
-            # if hasattr(v, "get_field_options"):
-            if issubclass(v.__class__, FieldElement):
-                result.update(field_options=v.get_field_options())
-                result.update(help_text=v.field.help_text)
-
-            return result
-        if isinstance(v, LayoutHandle):
-            # Layout entry-point
-            return dict(main=v.main,
-                        window_size=v.layout.window_size)
 
         if isinstance(v, Action):
             # todo include all aux info
@@ -314,6 +352,14 @@ class Renderer(JsRenderer, JsCacheRenderer):
                           http_method=v.http_method,
                           )
 
+            href_target = v.get_a_href_target()
+            if href_target:
+                result.update(href=href_target)
+
+            if isinstance(v, QuickLinkAction):
+                result['an'] = v.bound_action.action.action_name
+                result['actor'] = str(v.bound_action.actor)
+
             # if v.show_in_bbar: result["bbar"] = True # not needed
             if v.preprocessor: result["preprocessor"] = v.preprocessor
             if v.combo_group: result["combo_group"] = v.combo_group
@@ -326,113 +372,24 @@ class Renderer(JsRenderer, JsCacheRenderer):
 
             return result
 
-        if isinstance(v, BoundAction):
-
-            # todo include all aux info
-            # todo include grid info
-            # todo refactor this into a all_actions object and have the bound actions ref it to reduse json size
-            result = dict(an=v.action.action_name,
-                          window_layout=v.get_layout_handel(),
-                          )
-
-            # if v.action.show_in_bbar: result["bbar"] = True # not needed
-            if v.action.window_type:
-                tba = []
-                combo_group = None
-                for ba in v.actor.get_toolbar_actions(v.action):
-                    if ba.action.combo_group == combo_group and combo_group is not None:
-                        previous = tba.pop()
-                        if not isinstance(previous, list):
-                            previous = [previous]
-                        previous.append(ba.action.action_name)
-                        tba.append(previous)
-                    else:  # is normal
-                        tba.append(ba.action.action_name)
-                        combo_group = ba.action.combo_group
-                result["toolbarActions"] = tba
-
-            return result
-        if isclass(v) and issubclass(v, Actor):
-            result = dict(id=v.actor_id,
-                          ba=v._actions_dict,
-                          label=v.get_actor_label(),
-                          slave=bool(v.master),
-                          editable=v.editable,
-                          # [py2js(b) for b in v.actions.items()]
-                          )
-            # grids
-            if hasattr(v.get_handle(), "get_columns"):
-                result['col'] = v.get_handle().get_columns()
-                index_mod = 0
-                for c in result['col']:
-                    c.fields_index = find(v.get_handle().store.list_fields, c.field.name,
-                                          key=lambda f: f.name) + index_mod
-                    if isinstance(c, ComboFieldElement) and not isinstance(c, SimpleRemoteComboFieldElement):
-                        # Skip the data value for multi value columns, such as choices and FK fields.
-                        # use c.fields_index -1 for data value
-                        c.fields_index_hidden = c.fields_index + 1
-                        index_mod += 1
-
-            result.update(obj2dict(v.get_handle().store, "pk_index"))  # Data index which is the PK
-            result.update(obj2dict(v, "preview_limit "  # number of rows to render # if 0 no paginator.
-                                      "use_detail_param_panel "  # show PV panel in detail
-                                      "use_detail_params_value "  # in grid, use parrent PV values
-                                      "hide_top_toolbar "  # No selection and toolbar
-                                      "use_detail_params_value "
-                                      "react_responsive "
-                                      "react_big_search "
-                                      "display_mode "
-                                      "max_render_depth "
-                                      "simple_slavegrid_header "
-                                      "paginator_template "
-                                      "hide_if_empty "
-                                      "borderless_list_mode "
-                                   ))
-
-            card_layout = getattr(v, "card_layout", None)
-            if card_layout is not None:
-                result.update(card_layout=card_layout.get_layout_handle())
-            # mt + slave-tables
-
-            chooser_dict = getattr(v.model, "_choosers_dict", {})
-            if chooser_dict:
-                result.update(chooser_dict={fn: [cf.name for cf in c.context_fields]
-                                            for fn, c in chooser_dict.items()})
-
-            if settings.SITE.is_installed('contenttypes') and getattr(v, 'model', None) is not None and hasattr(v.model,
-                                                                                                                "_meta"):
-                # Perhaps I should have the model also be py2js'd?
-                result.update(content_type=ContentType.objects.get_for_model(v.model).pk)
-            for a in "detail_action insert_action default_action".split(" "):
-                if hasattr(v, a) and getattr(v, a) is not None:
-                    result.update({a: getattr(v, a).action.action_name})
-
-            if v.params_layout is not None:
-                # Param field array and layout struct
-                result.update(
-                    pv_layout=v.params_layout.get_layout_handle(),
-                    pv_fields=[f.name for f in v.params_layout.params_store.param_fields],
-                )
-            return result
-
         if isinstance(v, js_code) and getattr(self, "serialise_js_code", False):
             # Convert js_code into strings so they are serialised. rather than displayed w/o quotes
             return str(v.s)
 
         return v
 
-    def goto_instance(self, ar, obj, detail_action=None, **kw):
-        """Ask the client to display a :term:`detail window` on the given
-        record. The client might ignore this if Lino does not know a
-        detail window.
-
-        This calls :meth:`obj.get_detail_action
-        <lino.core.model.Model.get_detail_action>`.
-
-        """
-        js = self.instance_handler(ar, obj, detail_action)
-        kw.update(eval_js=js)
-        ar.set_response(**kw)
+    # def goto_instance(self, ar, obj, detail_action=None, **kw):
+    #     """Ask the client to display a :term:`detail window` on the given
+    #     record. The client might ignore this if Lino does not know a
+    #     detail window.
+    #
+    #     This calls :meth:`obj.get_detail_action
+    #     <lino.core.model.Model.get_detail_action>`.
+    #
+    #     """
+    #     js = self.instance_handler(ar, obj, detail_action)
+    #     kw.update(eval_js=js)
+    #     ar.set_response(**kw)
 
     def handler_item(self, mi, handler, help_text):
         """"""
@@ -455,9 +412,9 @@ class Renderer(JsRenderer, JsCacheRenderer):
         st = ar.get_status(**kw)
         return self.action_call(ar, ar.bound_action, st)
 
-    def instance_handler(self, ar, obj, ba):
-        # Used for navigation, currently working, due to action_call override
-        return super(Renderer, self).instance_handler(ar, obj, ba)
+    # def instance_handler(self, ar, obj, ba):
+    #     # Used for navigation, currently working, due to action_call override
+    #     return super(Renderer, self).instance_handler(ar, obj, ba)
 
     def action_call(self, request, bound_action, status):
 
@@ -492,49 +449,6 @@ class Renderer(JsRenderer, JsCacheRenderer):
         js = escape(js, quote=False)
         return 'javascript:' + js
 
-    def show_menu(self, ar, mnu, level=1):
-        """
-        Render the given menu as an HTML element.
-        Used for writing test cases.
-        """
-        if not isinstance(mnu, Menu):
-            assert isinstance(mnu, MenuItem)
-            if mnu.bound_action:
-                sar = mnu.bound_action.actor.request(
-                    action=mnu.bound_action,
-                    user=ar.user, subst_user=ar.subst_user,
-                    requesting_panel=ar.requesting_panel,
-                    renderer=self, **mnu.params)
-                # print("20170113", sar)
-                url = sar.get_request_url()
-            else:
-                url = mnu.href
-            assert mnu.label is not None
-            if url is None:
-                return E.p()  # spacer
-            return E.li(E.a(mnu.label, href=url, tabindex="-1"))
-
-        items = [self.show_menu(ar, mi, level + 1) for mi in mnu.items]
-        # ~ print 20120901, items
-        if level == 1:
-            return E.ul(*items, **{'class': 'nav navbar-nav'})
-        if mnu.label is None:
-            raise Exception("%s has no label" % mnu)
-        if level == 2:
-            cl = 'dropdown'
-            menu_title = E.a(
-                str(mnu.label), E.b(' ', **{'class': "caret"}), href="#",
-                data_toggle="dropdown", **{'class': 'dropdown-toggle'})
-        elif level == 3:
-            menu_title = E.a(str(mnu.label), href="#")
-            cl = 'dropdown-submenu'
-        else:
-            raise Exception("Menu with more than three levels")
-        return E.li(
-            menu_title,
-            E.ul(*items, **{'class': 'dropdown-menu'}),
-            **{'class': cl})
-
     def add_help_text(self, kw, help_text, title, datasource, fieldname):
         if settings.SITE.use_quicktips:
             if settings.SITE.show_internal_field_names:
@@ -560,6 +474,127 @@ class Renderer(JsRenderer, JsCacheRenderer):
         filename += translation.get_language() + '.' + file_type
         return ('cache', file_type, filename)
 
+    def actions2json(self, actions_list):  # 20210517
+        # v is a dict of an -> BoundCction instances
+        result = []
+
+        for ba in actions_list:
+            lh = ba.get_layout_handel()
+            if (lh or ba.action.window_type):
+                # and k not in [
+                #     "default_action", "insert_action", "detail_action"]:
+
+                action_descriptor = {'an': ba.action.action_name}
+                if lh:
+                    action_descriptor.update(window_layout=lh.layout._formpanel_name)
+                if ba.action.window_type:
+                    tba = []
+                    combo_group = None
+
+                    for a in ba.actor.get_toolbar_actions(
+                        ba.action, get_user_profile()):
+                        if a.action.combo_group == combo_group\
+                            and combo_group is not None:
+                            previous = tba.pop()
+                            if not isinstance(previous, list):
+                                previous = [previous]
+                            previous.append(a.action.action_name)
+                            tba.append(previous)
+                        else:  # is normal
+                            tba.append(a.action.action_name)
+                            combo_group = a.action.combo_group
+                    action_descriptor["toolbarActions"] = tba
+                result.append(action_descriptor)
+
+        return result
+
+    def actor2json(self, v):
+        assert isclass(v) and issubclass(v, Actor)
+        result = dict(id=v.actor_id,
+                      actions_list=self.actions2json(v._actions_list),
+                      label=v.get_actor_label(),
+                      slave=bool(v.master),
+                      editable=not v.hide_editing(get_user_profile())
+                      # [py2js(b) for b in v.actions.items()]
+                      )
+        ah = v.get_handle()
+        # grids
+        if hasattr(ah, "get_columns"):
+            columns = []
+            index_mod = 0
+            col_elems = ah.get_columns()
+            for col in col_elems:
+                d = self.elem2json(col)
+                d.update(fields_index = find(ah.store.list_fields, col.field.name,
+                                      key=lambda f: f.name) + index_mod)
+                if isinstance(col, ComboFieldElement) and not isinstance(col, SimpleRemoteComboFieldElement):
+                    # Skip the data value for multi value columns, such as choices and FK fields.
+                    # use c.fields_index -1 for data value
+                    d.update(fields_index_hidden = d['fields_index'] + 1)
+                    index_mod += 1
+                columns.append(d)
+            result['col'] = columns
+            # (dict(main=self.elem2json(col_elems.main),
+            #         col=columns,
+            #        window_size=col_layout.layout.window_size))
+
+        result.update(obj2dict(ah.store, "pk_index"))  # Data index which is the PK
+        result.update(obj2dict(v, "preview_limit "  # number of rows to render # if 0 no paginator.
+                                  "use_detail_param_panel "  # show PV panel in detail
+                                  "use_detail_params_value "  # in grid, use parrent PV values
+                                  "hide_navigator "
+                                  "use_detail_params_value "
+                                  "react_responsive "
+                                  "display_mode "
+                                  "max_render_depth "
+                                  "simple_slavegrid_header "
+                                  "paginator_template "
+                                  "hide_top_toolbar "
+                                  "hide_if_empty "
+                                  "borderless_list_mode "
+                               ))
+
+        if hasattr(v.model, "file"):
+            result.update(contain_media=True)
+
+        if v.detail_action is not None:
+            wl = v.detail_action.get_window_layout()
+            if wl is not None:
+                result.update(window_layout=wl._formpanel_name)
+
+        card_layout = getattr(v, "card_layout", None)
+        if card_layout is not None:
+            result.update(card_layout=card_layout._formpanel_name)
+
+        item_layout = getattr(v, "item_layout", None)
+        if item_layout is not None:
+            result.update(item_layout=item_layout._formpanel_name)
+        # mt + slave-tables
+
+        chooser_dict = getattr(v.model, "_choosers_dict", {})
+        if chooser_dict:
+            result.update(chooser_dict={fn: [cf.name for cf in c.context_fields]
+                                        for fn, c in chooser_dict.items()})
+
+        if settings.SITE.is_installed('contenttypes') and getattr(v, 'model', None) is not None and hasattr(v.model,
+                                                                                                            "_meta"):
+            # Perhaps I should have the model also be py2js'd?
+            result.update(content_type=ContentType.objects.get_for_model(v.model).pk)
+        for k in ("detail_action", "insert_action", "default_action"):
+            ba = getattr(v, k, None)
+            if ba is not None:
+                result.update({k: ba.action.action_name})
+
+        if v.params_layout is not None:
+            # Param field array and layout struct
+            result.update(
+                params_layout=v.params_layout._formpanel_name,
+                params_fields=[f.name for f in v.params_layout.params_store.param_fields],
+            )
+            if hasattr(v, "params_panel_pos"):
+                result.update(params_panel_pos=v.params_panel_pos)
+        return result
+
     def build_js_cache(self, force):
         self.serialise_js_code = True
 
@@ -568,10 +603,11 @@ class Renderer(JsRenderer, JsCacheRenderer):
 
             def write(f):
                 f.write(
-                    py2js(actor)
+                    py2js(self.actor2json(actor))
                 )
 
             settings.SITE.kernel.make_cache_file(fn, write, force)
+
         self.serialise_js_code = False
 
         return super(Renderer, self).build_js_cache(force)
