@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2009-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """Views for `lino_react.react`.
 """
@@ -18,8 +18,9 @@ from django import http
 from django.db import models
 from django.conf import settings
 from django.views.generic import View
+from django.views.generic.base import TemplateView
 from django.core import exceptions
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from django.utils.translation import get_language
 from django.core.exceptions import PermissionDenied
 
@@ -38,6 +39,7 @@ from lino.core.views import requested_actor, action_request
 from lino.core.views import json_response, json_response_kw
 from lino.core.views import choices_response
 from lino.core.utils import navinfo
+from lino.core.actions import ShowEmptyTable
 from lino.core import kernel
 
 from lino.modlib.users.utils import get_user_profile, with_user_profile
@@ -65,10 +67,12 @@ NOT_FOUND = "%s has no row with primary key %r"
 #     def get(self, request, thread_id, button_id):
 #         return settings.SITE.kernel.run_callback(request, thread_id, button_id)
 
+
 class ApiElement(ApiElement):
 
     def post(self, request, app_label=None, actor=None, pk=None):
-        data = http.QueryDict(request.body)
+        # data = http.QueryDict(request.body)
+        data = request.POST
         ar = action_request(
             app_label, actor, request, data, True,
             renderer=settings.SITE.plugins.react.renderer)
@@ -83,7 +87,6 @@ class ApiElement(ApiElement):
         data = http.QueryDict(request.body)  # raw_post_data before Django 1.4
         # data = json.loads(request.body)
         # logger.info("20150130 %s", data)
-
         ar = action_request(
             app_label, actor, request, data, False,
             renderer=settings.SITE.plugins.react.renderer)
@@ -98,13 +101,6 @@ class ApiElement(ApiElement):
         ar.set_selected_pks(pk)
         return settings.SITE.kernel.run_action(ar)
 
-    def old_delete(self, request, app_label=None, actor=None, pk=None):
-        rpt = requested_actor(app_label, actor)
-        ar = rpt.request(request=request)
-        ar.set_selected_pks(pk)
-        elem = ar.selected_rows[0]
-        return delete_element(ar, elem)
-
 
 class ApiList(View):
     def post(self, request, app_label=None, actor=None):
@@ -139,13 +135,20 @@ class ApiList(View):
             return settings.SITE.kernel.run_action(ar)
 
         if fmt == constants.URL_FORMAT_JSON:
+            if isinstance(ar.bound_action.action, ShowEmptyTable):
+                elem = ar.create_instance()
+                datarec = elem2rec_empty(ar, ar.ah, elem)
+                return json_response(datarec)
 
-            window_type = request.GET.get(constants.URL_PARAM_WINDOW_TYPE, "g")
+            window_type = request.GET.get(constants.URL_PARAM_WINDOW_TYPE, "t")
 
             def serialize(ar, row):
-                """Use window_type to determin which serizlisation store metod and fields to use"""
-                if window_type == "g":
+                """Use window_type to determine which serilisation store metod and fields to use"""
+                if window_type == "t":
                     return rh.store.row2list(ar, row)
+                elif window_type == "g":
+                    # return rh.store.file2url(ar, row)
+                    return row.get_gallery_item(ar)
                 else:
                     return rh.store.row2dict(ar, row,
                                              fields=rh.store.card_fields if window_type == "c" else rh.store.detail_fields,
@@ -156,7 +159,7 @@ class ApiList(View):
                     for row in ar.sliced_data_iterator]
 
             total_count = ar.get_total_count()
-            if window_type == "g":
+            if window_type == "t":
                 for row in ar.create_phantom_rows():
                     if ar.limit is None or len(rows) + 1 < ar.limit or ar.limit == total_count + 1:
                         d = serialize(ar, row)
@@ -168,7 +171,7 @@ class ApiList(View):
                       success=True,
                       no_data_text=ar.no_data_text,
                       title=ar.get_title())
-            if window_type != "g":
+            if window_type != "t":
                 mc = ar.get_main_card()
                 if mc is not None:
                     rows.insert(0, mc)
@@ -292,8 +295,7 @@ class Restful(View):
         """
         rpt = requested_actor(app_label, actor)
 
-        action_name = request.GET.get(constants.URL_PARAM_ACTION_NAME,
-                                      rpt.default_elem_action_name)
+        action_name = request.GET.get(constants.URL_PARAM_ACTION_NAME, None)
         fmt = request.GET.get(
             constants.URL_PARAM_FORMAT, constants.URL_FORMAT_JSON)
         sr = request.GET.getlist(constants.URL_PARAM_SELECTED)
@@ -310,8 +312,10 @@ class Restful(View):
             return json_response(kw)
 
         else:  # action_name=="detail": #ba.action.opens_a_window:
-
-            ba = rpt.get_url_action(action_name)
+            if action_name:
+                ba = rpt.get_url_action(action_name)
+            else:
+                ba = rpt.detail_action
             ah = ar.ah
             ar = ba.request(request=request, selected_pks=sr)
             elem = ar.selected_rows[0]
@@ -434,6 +438,18 @@ def XML_response(ar, tplname, context):
     return response
 
 
+class SWView(TemplateView):
+    template_name = 'react/service-worker.js'
+    content_type = 'application/javascript'
+
+class WBView(TemplateView):
+    content_type = 'application/javascript'
+
+    def setup(self, request, *args, **kwargs):
+        super().setup(request, *args, **kwargs)
+        self.template_name = 'react/' + kwargs['workbox']
+
+
 class MainHtml(View):
     def get(self, request, *args, **kw):
         """Returns a json struct for the main user dashboard."""
@@ -444,7 +460,7 @@ class MainHtml(View):
         # Add to reqeust not ar, as there's error if passing ar to get_main_html
         request.requesting_panel = "dashboard-main"
         html = settings.SITE.get_main_html(
-            request, extjs=settings.SITE.plugins.react)
+            request, front_end=settings.SITE.plugins.react)
         html = settings.SITE.plugins.react.renderer.html_text(html)
         ar.success(html=html)
         return json_response(ar.response, ar.content_type)
@@ -458,7 +474,10 @@ class DashboardItem(View):
         ar.requesting_panel = f"dashboard-{index}"
         dash = ar.get_user().get_preferences().dashboard_items
         if len(dash) > index:
-            html = ar.show_story([dash[index]])
+            item = dash[index]
+            # print("20210112 DashboardItem.get()", index, item.actor)
+            # html = ar.show_story([dash[index]])
+            html = ''.join(item.render(ar))
         else:
             html = ""
         ar.success(html=html)
@@ -611,13 +630,3 @@ class UserSettings(View):
             return json_response(user_settings)
 
         return with_user_profile((su or u).user_type, getit)
-
-
-class Suggestions(View):
-    def get(self, request, app_label=None, actor=None, pk=None, field=None):
-        suggesters = settings.SITE.plugins.memo.parser.suggesters
-        trigger = request.GET.get("trigger")
-        query = request.GET.get("query")
-        return json_response(
-            {"suggestions": list(suggesters[trigger].get_suggestions(query))}
-        )
