@@ -1,20 +1,20 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2015-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2015-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 SETUP_INFO = dict(
     name='lino_react',
-    version='20.12.0',
+    version='21.6.0',
     install_requires=['lino'],
     tests_require=[],
     test_suite='tests',
-    description="The React user interface for Lino",
-    license='BSD-2-Clause',
+    description="The React front end for Lino",
+    license_files=['COPYING'],
     include_package_data=False,
     zip_safe=False,
     author='Rumma & Ko Ltd',
     author_email='info@lino-framework.org',
-    url="http://www.lino-framework.org",
+    url="https://gitlab.com/lino-framework/react",
     classifiers="""\
   Programming Language :: Python
   Programming Language :: Python :: 3
@@ -23,10 +23,8 @@ SETUP_INFO = dict(
   Framework :: Django
   Intended Audience :: Developers
   Intended Audience :: System Administrators
-  License :: OSI Approved :: BSD License
+  License :: OSI Approved :: GNU Affero General Public License v3
   Natural Language :: English
-  Natural Language :: French
-  Natural Language :: German
   Operating System :: OS Independent
   Topic :: Database :: Front-Ends
   Topic :: Home Automation
@@ -37,7 +35,7 @@ SETUP_INFO.update(long_description="""\
 
 The React front end for Lino
 
-The central project homepage is http://react.lino-framework.org/
+Project homepage is https://gitlab.com/lino-framework/react
 
 
 """)
